
from setuptools import setup

import os
import re
import sys


if sys.version_info < (3, 4):
    sys.exit('Python < 3.4 is not supported')


# get abs path from this folder name
here = os.path.dirname(os.path.abspath(__file__))

# open __init__.py, where version is specified
with open(os.path.join(here, 'crepobenchmark', '__init__.py')) as f:
    txt = f.read()

# try to read it from source code
try:
    version = re.findall(r"^__version__ = '([^']+)'\r?$",
                         txt, re.M)[0]
except IndexError:
    raise RuntimeError('Unable to determine version.')



# function to read requirements, and include them as package dependencies
def get_requirements(*files):
    # read requirements.txt file and return them as a list of strings
    reqs = []
    for file in files:
        with open(file) as f:
            req = f.readlines()
        reqs += [r.strip() for r in req]
    return reqs  # clean lines from blank spaces and line breaks


setup(
    name='crepobenchmark',
    version=version,
    description='Benchmark for inference of credal networks.',
    long_description='Benchmark for inference of credal networks.',
    #long_description_content_type="text/markdown",
    author='Rafael Cabañas, Alessandro Antonucci',
    author_email="rcabanas@idsia.ch, alessandro@antonucci.ch",
    url='',
    #download_url='https://github.com/.../archive/{}.tar.gz'.format(version),
    keywords='credal networks',
    license='GNU GPLv3',
    classifiers=['Intended Audience :: Developers',
                 'Intended Audience :: Education',
                 'Intended Audience :: Science/Research',
                 'License :: OSI Approved :: Apache Software License',
                 'Operating System :: POSIX :: Linux',
                 'Operating System :: MacOS :: MacOS X',
                 'Operating System :: Microsoft :: Windows',
                 'Programming Language :: Python :: 3.4'],
    packages=['crepobenchmark'],
    python_requires='>=3.5',
    install_requires=['pandas~=1.1.0'],
    # extras_require={
    #     'gpu': get_requirements('requirements/gpu.txt'),
    #     'visualization': get_requirements('requirements/visualization.txt'),
    #     'datasets': get_requirements('requirements/datasets.txt'),
    #     'all': get_requirements('requirements/visualization.txt', 'requirements/datasets.txt'),
    #     'all-gpu': get_requirements('requirements/gpu.txt', 'requirements/visualization.txt', 'requirements/datasets.txt'),
    #
    # },
    # tests_require=get_requirements('requirements/test.txt'),
    include_package_data=True,
)


import os
from pathlib import Path

from crepobenchmark.gitconnection import request
from crepobenchmark.utils import singleton, write_file, gettempdir, exec_bash
import io


import pandas as pd


@singleton
class metainfo:
    local_jar = None
    local_df = None

    def reset(self):
        self.local_jar = None
        self.local_df = None

info = metainfo()

def download_metadata():
    info.reset()
    get_benchmark_data()
    save_jar()


def get_benchmark_data(reload:bool=False) -> pd.DataFrame:
    if reload: info.local_df = None
    if info.local_df is None:
        info.local_df = pd.read_csv(io.StringIO(request("data/full.csv")))
    return info.local_df

def save_jar(folder: str = None, reload:bool=False):
    if reload: info.local_jar = None
    if info.local_jar is None:
        folder = folder or gettempdir()
        content = request("src/java/lib/crepo.jar?raw=true", False)
        info.local_jar = os.path.join(Path(folder), Path(f"crepo.jar"))
        write_file(info.local_jar, content, True)

def get_model(label:str) -> str:
    return request(get_filename(label)[1:], decode=True)

def get_filename(label:str) -> str:
    return info.local_df[info.local_df["label"]==label]["filename"].iloc[0]


def save_model(label, filename):
    content = get_model(label)
    write_file(filename, content, False)


def describe():

    model_features = ['num_vert', 'max_degree', 'max_indegree', 'max_values',
                      'nodes', 'method', 'kind', 'query_type']
    data = info.local_df

    descr = [dict(property=f, value=list(pd.unique(data[f])))
             for f in model_features]

    vmodels = pd.unique(data[data["label"].str.startswith("v")]["label"]).shape[0]
    hmodels = pd.unique(data[data["label"].str.startswith("h")]["label"]).shape[0]

    descr.append(dict(property="Vmodels", value=vmodels))
    descr.append(dict(property="Hmodels", value=hmodels))
    descr.append(dict(property="rows", value=len(data)))
    descr.append(dict(property="columns", value=data.shape[-1]))


    df = pd.DataFrame(descr)

    text_descr = dict(
        num_vert = "Number of vertices in the credal sets. It is always 2 for binary variables.",
        max_degree = "Maximum arc degree in the model.",
        max_indegree = "Maximum arc indegree in the model.",
        max_values = "Maximum cardinality plus 1",
        nodes = "Number of nodes in the model.",
        method = "Inference method.",
        kind = "Topology of the DAG: singly or multy connected.",
        query_type = "Marginal or conditional query.",
        Vmodels = "Count of models with a vertex specification.",
        Hmodels = "Count of models with a linear constraints specification",
        rows = "Number of rows in the current benchmarking data.",
        columns = "Number of rows in the current benchmarking data.",
        )


    df["description"] = df.apply(lambda t:  text_descr[t["property"]], axis=1)
    return df


def run_crema(filename: str, target: int = 0, observed: str = "1", method: str = "cve",
              measure_time: bool = True, runs: int = 1, warmups: int = 0, timeout: int = 600, logfile: str = None,
              **kwargs):

    save_jar()
    observed = str(observed)
    javafile = f"ch.idsia.RunCrema"
    cmd = "-t" if measure_time else ""
    cmd += f" --method={method} -r {int(runs)} -w {int(warmups)} -x {int(target)}"
    if len(observed) > 0: cmd += f" -y {observed}"
    cmd += f" --timeout={timeout} {filename}"
    cmd += f" --log={logfile}" if logfile is not None else ""
    print(cmd)
    cmd = f"java -cp {info.local_jar} {javafile} {cmd}"
    output = exec_bash(cmd)
    exec(output[-1])
    return locals()["results"]


if __name__ == "__main__":
    download_metadata()
    data = get_benchmark_data()

    modelname = data.iloc[0]["label"]

    print(get_model(modelname))

    save_model(modelname, "model.uai")

    res = run_crema("model.uai", target=0)



import subprocess
import tempfile
from pathlib import Path




def singleton(class_):
    """ wrapper that allows to define a singleton class """

    class class_w(class_):
        _instance = None

        def __new__(class_, *args, **kwargs):
            if class_w._instance is None:
                class_w._instance = super(class_w,
                                          class_).__new__(class_,
                                                          *args,
                                                          **kwargs)
                class_w._instance._sealed = False
            return class_w._instance

        def __init__(self, *args, **kwargs):
            if self._sealed:
                return
            super(class_w, self).__init__(*args, **kwargs)
            self._sealed = True

    class_w.__name__ = class_.__name__
    return class_w



def gen_exec(cmd, check_return: bool = False):
    popen = subprocess.Popen(cmd, stdout=subprocess.PIPE, universal_newlines=True)
    for stdout_line in iter(popen.stdout.readline, ""):
        yield stdout_line
    popen.stdout.close()
    return_code = popen.wait()
    if return_code and check_return:
        raise subprocess.CalledProcessError(return_code, cmd)


def exec_bash(cmd: str, check_return: bool = False):
    return [s for s in gen_exec(cmd.split(), check_return)]


def exec_bash_print(cmd: str, check_return: bool = False):
    for path in gen_exec(cmd.split(), check_return):
        print(path, end="")


def gettempdir():
    return tempfile.gettempdir()


def write_file(path, content, binary:bool = False):
    flags = "wb+" if binary else "w+"
    with open(Path(path).absolute(), flags) as f:
        f.write(content)


import requests

from crepobenchmark import __version__ as vtag

from crepobenchmark.utils import singleton


@singleton
class GitConnection(object):

    def __init__(self):
        self._base_url = f"https://raw.githubusercontent.com/IDSIA-papers/crepo/{vtag}/"

    @property
    def base_url(self):
        return self._base_url

# singleton instance
git_connect = GitConnection()

def request(remote_path:str, decode:bool = True):
    url = git_connect._base_url+remote_path
    s = requests.get(url)
    if not s.ok:
        raise ConnectionError(f"Error downloading {url}. Status code {s.status_code}")
    return s.content.decode('utf-8') if decode else s.content

def check():
    request = requests.get(git_connect._base_url + "README.md")
    return request.status_code == 200

